﻿// Commands.cs
//

using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;

namespace HelloWorld {

    public static class Commands {

        private static readonly DependencyProperty CommandBehaviorProperty =
            DependencyProperty.RegisterAttached("CommandButtonBehavior", typeof(CommandButtonBehavior), typeof(Commands), null);

        public static readonly DependencyProperty CommandProperty =
            DependencyProperty.RegisterAttached("Command", typeof(ICommand), typeof(Commands), new PropertyMetadata(CommandPropertyChanged));

        public static readonly DependencyProperty CommandParameterProperty =
            DependencyProperty.RegisterAttached("CommandParameter", typeof(object), typeof(Commands), null);

        public static ICommand GetCommand(ButtonBase buttonBase) {
            return (ICommand)buttonBase.GetValue(CommandProperty);
        }

        public static object GetCommandParameter(ButtonBase buttonBase) {
            return buttonBase.GetValue(CommandParameterProperty);
        }

        public static void SetCommand(ButtonBase buttonBase, ICommand value) {
            buttonBase.SetValue(CommandProperty, value);
        }

        public static void SetCommandParameter(ButtonBase buttonBase, object value) {
            buttonBase.SetValue(CommandParameterProperty, value);
        }

        private static void CommandPropertyChanged(DependencyObject o, DependencyPropertyChangedEventArgs e) {
            ButtonBase element = o as ButtonBase;
            if (element != null) {
                if (e.OldValue != null) {
                    UnhookCommand(element, (ICommand)e.OldValue);
                }
                if (e.NewValue != null) {
                    HookCommand(element, (ICommand)e.NewValue);
                }
            }
        }

        private static void HookCommand(ButtonBase element, ICommand command) {
            CommandButtonBehavior behavior = new CommandButtonBehavior(element, command);
            behavior.Attach();
            element.SetValue(CommandBehaviorProperty, behavior);
        }

        private static void UnhookCommand(ButtonBase element, ICommand command) {
            CommandButtonBehavior behavior = (CommandButtonBehavior)element.GetValue(CommandBehaviorProperty);
            behavior.Detach();
            element.ClearValue(CommandBehaviorProperty);
        }


        private class CommandButtonBehavior {

            private readonly WeakReference elementReference;
            private readonly ICommand command;

            public CommandButtonBehavior(ButtonBase element, ICommand command) {
                this.elementReference = new WeakReference(element);
                this.command = command;
            }

            public void Attach() {
                ButtonBase element = GetElement();
                if (element != null) {
                    element.Click += OnClick;
                    command.CanExecuteChanged += OnCommandCanExecuteChanged;
                    SetIsEnabled(element);
                }
            }

            public void Detach() {
                command.CanExecuteChanged -= OnCommandCanExecuteChanged;
                ButtonBase element = GetElement();
                if (element != null) {
                    element.Click -= OnClick;
                }
            }

            private ButtonBase GetElement() {
                return elementReference.Target as ButtonBase;
            }

            private static void OnClick(object sender, EventArgs e) {
                DependencyObject element = (DependencyObject)sender;
                ICommand command = (ICommand)element.GetValue(CommandProperty);
                object commandParameter = element.GetValue(CommandParameterProperty);
                command.Execute(commandParameter);
            }

            private void OnCommandCanExecuteChanged(object sender, EventArgs e) {
                ButtonBase element = GetElement();
                if (element != null) {
                    SetIsEnabled(element);
                }
                else {
                    Detach();
                }
            }

            private void SetIsEnabled(ButtonBase element) {
                element.IsEnabled = command.CanExecute(element.GetValue(Commands.CommandParameterProperty));
            }
        }
    }
}
