﻿// TweetListViewModel.cs
//

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows.Input;
using PatternsFramework;
using TwitterBug;
using TwitterBug.Services;

namespace TwitterBug.Models {

    public class TweetGroup : Group<string, Tweet> {

        private string _imageUrl;

        public TweetGroup(Tweet tweet, string screenName)
            : base(tweet, screenName) {
            _imageUrl = tweet.ImageUrl;
        }

        public string ImageUrl {
            get {
                return _imageUrl;
            }
        }

        public string ScreenName {
            get {
                return Key;
            }
        }
        
        public IEnumerable<Tweet> Messages {
            get {
                return this;
            }
        }
    }

    [Export]
    public class TweetListViewModel : ViewModelBase {

        private ITwitterService _twitterService;
        private IEventAggregator _eventAggregator;

        private DelegateCommand _refreshCommand;
        private IEnumerable<TweetGroup> _tweets;
        private bool _loading;

        [ImportingConstructor]
        public TweetListViewModel(ITwitterService twitterService, IEventAggregator eventAggregator) {
            _twitterService = twitterService;
            _eventAggregator = eventAggregator;
            _eventAggregator.Subscribe<TweetEventArgs>(OnTweetEvent);

            _refreshCommand = new DelegateCommand(Refresh, GetCanRefresh);
            LoadTweets();
        }

        public bool IsLoading {
            get {
                return _loading;
            }
            private set {
                _loading = value;
                _refreshCommand.RaiseCanExecuteChanged();
                RaisePropertyChanged("IsLoading");
            }
        }

        public ICommand RefreshCommand {
            get {
                return _refreshCommand;
            }
        }

        public IEnumerable<TweetGroup> Tweets {
            get {
                return _tweets;
            }
            set {
                _tweets = value;
                RaisePropertyChanged("Tweets");
            }
        }

        private bool GetCanRefresh() {
            return !_loading;
        }

        private void Refresh() {
            LoadTweets();
        }

        private void OnTweetEvent(TweetEventArgs e) {
            LoadTweets();
        }

        private void LoadTweets() {
            if (IsLoading) {
                return;
            }

            IsLoading = true;
            _twitterService.LoadTweets(TweetKind.Friends, OnTweetsLoaded, TweetKind.User);
        }

        private void OnTweetsLoaded(bool success, object userState) {
            if (success) {
                Tweets = _twitterService.Tweets.AsQueryable().
                            OrderByDescending(t => t.Date).
                            GroupByContiguous<Tweet, string, TweetGroup>(
                                t => t.ScreenName,
                                StringComparer.OrdinalIgnoreCase,
                                (t, s) => new TweetGroup(t, s));

                if (userState != null) {
                    if ((TweetKind)userState == TweetKind.User) {
                        _twitterService.LoadTweets(TweetKind.User, OnTweetsLoaded, TweetKind.Mentions);
                        return;
                    }
                    else if ((TweetKind)userState == TweetKind.Mentions) {
                        _twitterService.LoadTweets(TweetKind.Mentions, OnTweetsLoaded, null);
                        return;
                    }
                }
            }

            IsLoading = false;
            _refreshCommand.RaiseCanExecuteChanged();
        }
    }
}
